/*
 * STR - Support for explicit length strings. These have the advantage of
 *       allowing to pass around substrings without copying, to use the length
 *       as a quick hash, and to contain the \0 character.
 *
 * Author:
 * Emile van Bergen, emile@evbergen.xs4all.nl
 *
 * Permission to redistribute an original or modified version of this program
 * in source, intermediate or object code form is hereby granted exclusively
 * under the terms of the GNU General Public License, version 2. Please see the
 * file COPYING for details, or refer to http://www.gnu.org/copyleft/gpl.html.
 *
 * History:
 * 2004/01/13 - EvB - Created
 * 2004/05/11 - EvB - Split off from misc
 * 2005/03/23 - EvB - Copied from OpenRADIUS
 * 2005/05/23 - EvB - Added FREESTR macro and str_dup, str_safedup functions
 */


#ifndef _STR_H
#define _STR_H 1


/*
 * INCLUDES & DEFINES
 */


#include <sys/types.h>		/* for ssize_t */
#include <malloc.h>		/* for malloc, free */


#define FREESTR(str) do	{					\
    if ((str).p) { free((str).p); (str).p = 0; (str).l = 0; }	\
    } while (0)


/*
 * TYPES
 */


/* Semantics:
 * non-zero l with NULL p is illegal; the functions here don't check p if l>0
 * non-NULL p with zero l is legal; p will be freed by FREESTR
 * NULL p is legal if l is zero */

typedef struct str {
    char *p;
    ssize_t l;
} STR;


/*
 * PROTOTYPES
 */


void *memrchr(void *s, int c, size_t n);
size_t memspn(void *data, size_t len, char *set, int setlen);
size_t memcspn(void *data, size_t len, char *set, int setlen);

ssize_t cpstr(char *d, ssize_t dl, char *s, ssize_t sl);   /* copy string */
ssize_t cptstr(char *d, ssize_t dl, char *s, ssize_t sl);  /* copy, terminate */

void str_tolower(char *p, ssize_t l);
void str_toupper(char *p, ssize_t l);

STR str_getword(STR *s, char *delim, int delimcnt);

STR str_safedup(char *p, ssize_t l);		/* uses safe_malloc */
STR str_dup(char *p, ssize_t l);		/* may return NULL p, l */

#endif


/* 
 * vim:softtabstop=4:sw=4 
 */

